#ifndef _JS_CONSTANTS_H
#define _JS_CONSTANTS_H

#include "js_typedefs.h"

namespace js
{

class progression
{
	public:
		static const uint8 LRCP = 0;
		static const uint8 RLCP = 1;
		static const uint8 RPCL = 2;
		static const uint8 PCRL = 3;
		static const uint8 CPRL = 4;
};

class marker
{
	public:
		static const uint8 MSTART = 0xFF; // Start of marker

		// Delimiting markers and marker segments
		static const uint16 SOC = 0xFF4F; // Start of codestream
		static const uint16 SOT = 0xFF90; // Start of tile-part
		static const uint16 SOD = 0xFF93; // Start of data
		static const uint16 EOC = 0xFFD9; // End of codestream

		// Fixed information marker segments
		static const uint16 SIZ = 0xFF51; // Image and tile size

		// Functional marker segments
		static const uint16 COD = 0xFF52; // Coding style default
		static const uint16 COC = 0xFF53; // Coding style component
		static const uint16 QCD = 0xFF5C; // Quantization default
		static const uint16 QCC = 0xFF5D; // Quantization component
		static const uint16 RGN = 0xFF5E; // Region of interest
		static const uint16 POC = 0xFF5F; // Progression order change

		// Pointer marker segments
		static const uint16 PLM = 0xFF57; // Tile-part length
		static const uint16 PLT = 0xFF58; // Packet length, main header
		static const uint16 PPM = 0xFF60; // Packet length, tile-part header
		static const uint16 PPT = 0xFF61; // Packed packet headers, main header
		static const uint16 TLM = 0xFF55; // Packed packet headers, tile-part header

		// In-bit-stream markers and marker segments
		static const uint16 SOP = 0xFF91; // Start of packet
		static const uint16 EPH = 0xFF92; // End of packet header
		
		// Informational marker segments
		static const uint16 CRG = 0xFF63; // Component registration
		static const uint16 COM = 0xFF64; // Comment
};

} // End namespace js

#endif