#include <iostream>
#include <fstream>
#include <time.h>
#include <string>

#include "js_parse_funcs.h"

using namespace std;
using namespace js;

int main(int argc, char* argv[])
{
	bool human = false; // Set to true if human using program

	char *inFile, *outFile;
	uint32 target = 0;
	uint16 targetLayers = 0;
	
	if(argc != 9 && argc != 7)
	{
		std::cout << "Wrong amount of input parameters." << std::endl;
		if(human) cin.get();
		exit(0);
	}
	uint8 count = 0;
	for(int i = 1; i < argc-1; i++)
	{
		if(string(argv[i]) == "-i"){
			inFile = argv[i+1];
			count |= 1;
		}else if(string(argv[i]) == "-o"){
			outFile = argv[i+1];
			count |= 2;
		}else if(string(argv[i]) == "-b"){
			target = strtol(argv[i+1], NULL, 10);
			count |= 4;
		}else if(string(argv[i]) == "-l"){
			targetLayers = strtol(argv[i+1], NULL, 10);
			count |= 4;
		}
	}
	if(count != 7) // 4 lsb's set to 1
	{
		std::cout << "One of the input arguments are not set." << std::endl;
		if(human) cin.get();
		exit(0);
	}

	ifstream* inputImage = new ifstream(inFile, ios::binary | ios::in | ios::ate); // Open and move to the end
	ofstream* outputImage = new ofstream(outFile, ios::binary | ios::out | ios::trunc);

	ifstream::pos_type size;
	char* buffer_in;

	double diff = 0;
	uint32 sizeNew = 0;

	if(inputImage->is_open() && outputImage->is_open())
	{
		size = inputImage->tellg(); // We have set the pointer to the end of the file, so the current position is the size
		buffer_in = new char[uint32(size)];
		
		inputImage->seekg(0, ios::beg); // Go to the start of the file again
		inputImage->read(buffer_in, size); // Read the entire thing

		time_t start = clock();
		exit_res result = scale(buffer_in, uint32(size), sizeNew, target, targetLayers, human);
		diff = (clock()-start)/double(CLOCKS_PER_SEC);

		if(result == exit_success)
		{
			outputImage->write(buffer_in, sizeNew);
		}else{
			std::cout << "Parser failed." << std::endl;
		}

		inputImage->close();
		outputImage->close();
		delete[] buffer_in;
	}
	else
	{
		std::cout << "Could not open file." << std::endl;
	}
	
	delete inputImage;
	delete outputImage;

	if(human)
	{
		std::cout << "Time taken: " << diff << " seconds. " << std::endl;
		std::cout << "Press enter to quit...";
		cin.get();
	}
	
	return exit_success;
}