minMbit = 2*(512*512)/(1280*720);
maxMbit = 30*(512*512)/(1280*720);

nLayers = 5:80;
fps = 25;
minSSIM = 10;
minL = 0;
nLayersLength = length(nLayers);

imagemagick = 'C:\\Program Files\\ImageMagick-6.7.5-Q16\\convert.exe'; % Path to image magick (for conversion from png to bmp)
folder = '..\\TestImages\\'; % Name of folder with the test images
rawformat = 'bmp3';

files = dir(sprintf('%s/*.%s', folder, rawformat));
numFiles = numel(files);
origFileSizes = ones(numFiles,1);
origFileSizes(:) = files.bytes;
ssim_overhead_total = zeros(nLayersLength, 1);

%%
for k = 1:numFiles
    input = files(k).name;
    input = sprintf('%s%s', folder, input);

    % If input image is not bmp, convert it to bmp (version 3)
    if(~strcmp(rawformat, 'bmp3'))
        input2 = sprintf('%s.bmp3', strrep(input, '.png', ''));
        system(sprintf('\"%s\" %s %s',imagemagick,input,input2));
        input = input2;
    end
    files(k).name = input;
    s=dir(input);
    origFileSizes(k) = s.bytes;
end

%%
for l = 1:nLayersLength
    fprintf('Processing layer %u of %u\n', l, nLayersLength);

    % As long as all the file sizes are the same it's ok to put it here
    layers = calcLayers(nLayers(l), minMbit, maxMbit, origFileSizes(k), fps); % 20 layers in the range 5 to 200 Mbit/s at 50 fps where each frame is 6220854 bytes big.
    Lstring = '';
    for j = 1:nLayers(l)-2
       Lstring = strcat(Lstring, sprintf('%f', layers(j)));
       Lstring = strcat(Lstring, ',');
    end
    Lstring = strcat(Lstring, sprintf('%f', layers(nLayers(l)-1)));
    bitrates = layers*origFileSizes(k)-1; % One byte less should make us hit the worst case each time
    ssim_overhead = zeros(length(bitrates), 1);

   % fprintf('File number: ');
    for k = 1:numFiles
        %fprintf('%u,', k);
        [status, result] = system(sprintf('jasper --input %s --output out.jpc -O rate=%f -O ilyrrates=%s -O numrlvls=5 -O sop -O prg=lrcp', files(k).name, layers(nLayers(l)), Lstring)); 
        original = imread(files(k).name, 'bmp');
        for j = 1:length(bitrates)
            bytes = uint32(bitrates(j));
            rate = double(bytes)/origFileSizes(k);

            [status, result] = system(sprintf('Jp2parser -i out.jpc -o scaled.jpc -b %u', bytes)); 
            [status, result] = system(sprintf('jasper --input %s --output rated.jpc -O rate=%f -O numrlvls=5 -O prg=lrcp', files(k).name, rate));
            [status, result] = system('jasper --input scaled.jpc --output scaled.bmp --output-format bmp');
            [status, result] = system('jasper --input rated.jpc --output rated.bmp --output-format bmp');

            scaled = imread('scaled.bmp', 'bmp');
            rated = imread('rated.bmp', 'bmp');
            [ssim_scaled dummy] = ssim(original, scaled);
            [ssim_rated dummy] = ssim(original, rated);
            ssim_overhead(j, 1) = ssim_overhead(j, 1) + (1-ssim_scaled)/(1-ssim_rated);
        end
    end
    ssim_overhead(:,1) = ssim_overhead(:,1)./(numFiles);
    currmax = max(ssim_overhead);
    ssim_overhead_total(l) = currmax;
    if(currmax < minSSIM)
        minSSIM = currmax;
        minL = nLayers(l);
        fprintf('Current minimum DSSIM: %f at layer %u\n', minSSIM, minL);
    else
        fprintf('Current DSSIM (no update): %f at layer %u\n', currmax, minL);
    end

    %fprintf('\n');
end
%%

fprintf('Minimum DSSIM: %f\n at layer %u\n', minSSIM, minL);
h = figure(1);
plot(nLayers, ssim_overhead_total);
xlabel('Number of layers');
ylabel('Ratio of SSIM overhead');
title(sprintf('Overhead for different numbers of layers: %f to %f Mbits',minMbit, maxMbit));
print(h, '-djpeg', sprintf('opt%fto%f.jpg', minMbit, maxMbit));
%%


