%% Parameters
close all;
calcstuff = false;
minMbit = 2;
maxMbit = 30;
clipLength = 251; % length of the test clips
fps = 25;

ClipNames = {'ParkJoy', 'OldTownCross', 'InToTree', 'DucksTakeOff', 'CrowdRun'};
drops = maxMbit - [0 5 10 15 20 24 26 28]; % how much the bitrate will drop in the different tests

if(calcstuff)
    ssim_vals = zeros(5, 7, 2);
    psnr_vals = zeros(5, 7, 2);
    for i = 1:5
        currclip = char(ClipNames(i));
        for j = 2:8
            bytes = uint32(drops(j)*(1000000/(fps*8)));
            for k = 1:clipLength
                system(sprintf('Jp2parser -i ../SVT/720p25/%s/JPC/%03u.jpc -o Temp/scaled.jpc -b %u', currclip, k, bytes));
                system(sprintf('jasper --input ../SVT/720p25/%s/JPC/%03u.jpc --output Temp/ref.bmp --output-format bmp', currclip, k));
                system('jasper --input Temp/scaled.jpc --output Temp/scaled.bmp --output-format bmp');

                ref = imread('Temp/ref.bmp', 'bmp');
                scaled = imread('Temp/scaled.bmp', 'bmp');
                original = imread(sprintf('../SVT/720p25/%s/BMP/%03u.bmp', currclip, k), 'bmp');

                [ssim_val dummy] = ssim(ref, scaled);
                psnr_val = psnr(ref, scaled);
                ssim_vals(i, j-1, 1) = ssim_vals(i, j-1, 1) + ssim_val;
                psnr_vals(i, j-1, 1) = psnr_vals(i, j-1, 1) + psnr_val(1);

                [ssim_val dummy] = ssim(original, scaled);
                psnr_val = psnr(original, scaled);
                ssim_vals(i, j-1, 2) = ssim_vals(i, j-1, 2) + ssim_val;
                psnr_vals(i, j-1, 2) = psnr_vals(i, j-1, 2) + psnr_val(1);
            end
            ssim_vals(i, j-1, 1) = ssim_vals(i, j-1, 1)/clipLength;
            psnr_vals(i, j-1, 1) = psnr_vals(i, j-1, 1)/clipLength;
            ssim_vals(i, j-1, 2) = ssim_vals(i, j-1, 2)/clipLength;
            psnr_vals(i, j-1, 2) = psnr_vals(i, j-1, 2)/clipLength;
        end
    end
else
    load('ssim_verdier.mat');
    load('psnr_verdier.mat');
end

%% Plot stuff

%save('ssim_verdier.mat', 'ssim_vals');
%save('psnr_verdier.mat', 'psnr_vals');
close all;
for i = 1:5
    h = figure;
    stem(drops(2:8), psnr_vals(i, :, 1), 'black', 'filled', 'LineStyle','none');
    ylim([20 45]);
    xlim([0 30]);
    grid on;
    title(sprintf('PSNR (luma) for: %s', char(ClipNames(i))));
    xlabel('Bitrate [Mbits]');
    ylabel('PSNR');
    set(gca,'XDir','reverse');
    print(h, '-dpng', sprintf('SubtestResults/PSNRQRef_%s.png', char(ClipNames(i))));
    
    h = figure;
    stem(drops(2:8), ssim_vals(i, :, 1), 'black', 'filled', 'LineStyle','none');
    ylim([0.7 1]);
    xlim([0 30]);
    grid on;
    title(sprintf('SSIM for: %s', char(ClipNames(i))));
    xlabel('Bitrate [Mbits]');
    ylabel('SSIM');
    set(gca,'XDir','reverse');
    print(h, '-dpng', sprintf('SubtestResults/SSIMQRef_%s.png', char(ClipNames(i))));
end
%%
for i = 1:5
    h = figure;
    stem(drops(2:8), psnr_vals(i, :, 2), 'black', 'filled', 'LineStyle','none');
    ylim([20 45]);
    xlim([0 30]);
    grid on;
    title(sprintf('PSNR (luma) for: %s - compared to original', char(ClipNames(i))));
    xlabel('Bitrate [Mbits]');
    ylabel('PSNR');
    set(gca,'XDir','reverse');
    print(h, '-dpng', sprintf('SubtestResults/PSNRQOrig_%s.png', char(ClipNames(i))));
    
    h = figure;
    stem(drops(2:8), ssim_vals(i, :, 2), 'black', 'filled', 'LineStyle','none');
    ylim([0.7 1]);
    xlim([0 30]);
    grid on;
    title(sprintf('SSIM for: %s - compared to original', char(ClipNames(i))));
    xlabel('Bitrate [Mbits]');
    ylabel('SSIM');
    set(gca,'XDir','reverse');
    print(h, '-dpng', sprintf('SubtestResults/SSIMQOrig_%s.png', char(ClipNames(i))));
end




























