minMbit = 2*(512*512)/(1280*720);
maxMbit = 30*(512*512)/(1280*720);
nLayers = 3:45;
fps = 25;
nLayersLength = length(nLayers);
speeds = zeros(nLayersLength, 2);

imagemagick = 'C:\\Program Files\\ImageMagick-6.7.5-Q16\\convert.exe'; % path to image magick
folder = '..\\TestImages\\'; % folder with test images
rawformat = 'png';

files = dir(sprintf('%s/*.%s', folder, rawformat));
numFiles = numel(files);
origFileSizes = ones(numFiles,1);
origFileSizes(:) = files.bytes;

%%
for k = 1:numFiles
    input = files(k).name;
    input = sprintf('%s%s', folder, input);

    % If input image is not bmp, convert it to bmp (version 3)
    if(~strcmp(rawformat, 'bmp3'))
        input2 = sprintf('%s.bmp3', strrep(input, '.png', ''));
        system(sprintf('\"%s\" %s %s',imagemagick,input,input2));
        input = input2;
    end
    files(k).name = input;
    s=dir(input);
    origFileSizes(k) = s.bytes;
end

%%
for l = 1:nLayersLength
    fprintf('Processing layer %u of %u\n', l, nLayersLength);

    % As long as all the file sizes are the same it's ok to put it here
    layers = calcLayers(nLayers(l), minMbit, maxMbit, origFileSizes(k), fps); % 20 layers in the range 5 to 200 Mbit/s at 50 fps where each frame is 6220854 bytes big.
    Lstring = '';
    for j = 1:nLayers(l)-2
       Lstring = strcat(Lstring, sprintf('%f', layers(j)));
       Lstring = strcat(Lstring, ',');
    end
    Lstring = strcat(Lstring, sprintf('%f', layers(nLayers(l)-1)));

    % Layers
    timetot = 0;
    for k = 1:numFiles
        start = tic;
        [status, result] = system(sprintf('jasper --input %s --output out.jpc -O rate=%f -O ilyrrates=%s -O numrlvls=5 -O sop -O prg=lrcp', files(k).name, layers(nLayers(l)), Lstring)); 
        timetot = timetot + toc(start);
    end
    speeds(l, 1) = timetot/numFiles;
    
    
    % Precincts
    timetot = 0;
    for k = 1:numFiles
        start = tic;
        [status, result] = system(sprintf('jasper --input %s --output out.jpc -O rate=%f -O ilyrrates=%s -O numrlvls=5 -O sop -O prg=lrcp -O prcwidth=256 -O prcheight=256', files(k).name, layers(nLayers(l)), Lstring)); 
        timetot = timetot + toc(start);
    end
    speeds(l, 2) = timetot/numFiles;
    
end
h = figure(1);
plot(nLayers, speeds(:,1), 'b');
hold on;
plot(nLayers, speeds(:,2), 'r')
legend('1 precinct', '4 precincts');
xlabel('Total number of layers used');
ylabel('Seconds taken to encode');
title('Encoding time for bitrate range 2 to 30 Mbit');
print(h, '-djpeg', 'speeds2to30.jpg');


 