%% Parameters
encode = true;
nLayers = 40; % number of layers to use when encoding

minMbit = 2;
maxMbit = 30;
fps = 25;
uncompFileSize = 2764854; %size (in bytes) of one frame
clipLength = 251; % length of the test clips

name = 'Test720';
ClipNames = {'ParkJoy', 'OldTownCross', 'InToTree', 'DucksTakeOff', 'CrowdRun'};

drops = maxMbit - [0 5 10 15 20 24 26 28]; % how much the bitrate will drop in the different tests
duration = 2; % duration of drop in seconds

%% Initialize

nDrops = length(drops); % number of drops
nClips = length(ClipNames);
duration = duration*fps; % duration in number of frames

sequence = zeros(1, nDrops*nClips);
counter = 1;
for i = 1:nDrops
   for j = 1:nClips
      sequence(counter) = j*10 + i;
      counter = counter + 1;
   end
end
sequence = sequence(randperm(nClips*nDrops)); % scramble the test sequence
same = true;
while(same)
    same = false;
    for a = 1:length(sequence)-1
       if(floor(sequence(a)/10) == floor(sequence(a+1)/10))
            same = true;
            if(a+2 > length(sequence)) 
                temp = sequence(1);
                sequence(1) = sequence(a+1);
                sequence(a+1) = temp;
            else
                temp = sequence(a+2);
                sequence(a+2) = sequence(a+1);
                sequence(a+1) = temp;
            end
       end
    end
end

training = [11 22 33 44 55];
if(floor(training(5)/10) == floor(sequence(1)/10))
   training(5) = 15; 
end
sequence = [training sequence];

if matlabpool('size') == 0
    matlabpool open 7; % use 7 threads
end

close all;

%% Encode

if(encode)
    L = calcLayers(nLayers, minMbit, maxMbit, uncompFileSize, fps); 
    Lstring = '';
    for i = 1:nLayers-2
       Lstring = strcat(Lstring, sprintf('%f', L(i)));
       Lstring = strcat(Lstring, ',');
    end
    Lstring = strcat(Lstring, sprintf('%f', L(nLayers-1)));
    Lmax = L(nLayers);

    for n = 1:nClips
        currName = char(ClipNames(n));
        fprintf('Encoding %s\n', currName);
        parfor i = uint32(1:clipLength)
           [status, result] = system(sprintf('jasper --input ../SVT/720p25/%s/BMP/%03u.bmp --output ../SVT/720p25/%s/JPC/%03u.jpc -O rate=%f -O sop -O prg=lrcp -O numrlvls=5 -O ilyrrates=%s', currName, i, currName, i, Lmax, Lstring));
        end
    end
end

%% Make rate functions for the duration of the clips and Convert clips to the right bitrates using the parser
rate = ones(1, clipLength)*maxMbit; % max is the default
fid = fopen('TestInfo720.txt', 'wt'); % so we know what the test looks like afterwards
t = floor((clipLength/2) - (duration/2):(clipLength/2) + (duration/2) - 1); % Minus one because of the middle frame. Assumes even numbers.
frame_index = uint32(1);
system(sprintf('\"C:/Program Files/ImageMagick-6.7.5-Q16/convert.exe\" grey720.bmp grey720.jpc'));

for x = 1:length(sequence)
     fprintf('Generating clip %u\n', x);
    
    clip_index = floor(sequence(x)/10);
    drop_index = sequence(x) - clip_index*10;
    
    rate(t) = linspace(drops(drop_index), maxMbit, length(t));
    bytes = uint32(rate.*1000000./(8*fps));
    currName = char(ClipNames(clip_index));
  
    fprintf(fid, 'StartFrame: %u Mbits: %f Clip: %s\n', frame_index, drops(drop_index), currName);

    % Reference clip:
    parfor i = uint32(frame_index:frame_index+clipLength-1)
       [status, result] = system(sprintf('Jp2parser -i ../SVT/720p25/%s/JPC/%03u.jpc -o Temp/%05u.jpc -b %u', currName, i-frame_index+1, i, bytes(1))); % hacky, but works
    end
    frame_index = frame_index + clipLength;
    
    % 3 seconds mid grey
    parfor i = uint32(1:fps*3)
        system(sprintf('\"C:/Program Files/ImageMagick-6.7.5-Q16/convert.exe\" grey720.jpc Temp/%05u.jpc', frame_index+i-1));
    end
    frame_index = frame_index + fps*3;
    
    % Test clip
    parfor i = uint32(frame_index:frame_index+clipLength-1)
       [status, result] = system(sprintf('Jp2parser -i ../SVT/720p25/%s/JPC/%03u.jpc -o Temp/%05u.jpc -b %u', currName, i-frame_index+1, i, bytes(i-frame_index+1)));
    end
    frame_index = frame_index + clipLength;
    
    % 7 seconds mid grey
    system(sprintf('\"C:/Program Files/ImageMagick-6.7.5-Q16/convert.exe\" grey720.bmp -font Arial -pointsize 150 -gravity Center -fill white -annotate 0 %u greynum720.jpc', uint32(x)));
    parfor i = uint32(1:fps*7)
        system(sprintf('\"C:/Program Files/ImageMagick-6.7.5-Q16/convert.exe\" greynum720.jpc Temp/%05u.jpc', frame_index+i-1));
    end
    frame_index = frame_index + fps*7;
end
fclose(fid);


%% Save as avi file
system(sprintf('ffmpeg -f image2 -r 25 -i Temp/%%05d.jpc -pix_fmt yuv420p -vcodec rawvideo -an %s.avi', name));












